/**
 * Dream
 * Copyright (C) 2003-2004 INRIA Rhone-Alpes
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Contact: dream@objectweb.org
 *
 * Initial developer(s): Pierre Garcia
 * Contributor(s): 
 */

package org.objectweb.dream.dreamannotation.processor;

import org.objectweb.dream.dreamannotation.DreamComponent;
import org.objectweb.fractal.fraclet.annotation.annotations.Attribute;
import org.objectweb.fractal.fraclet.annotation.annotations.FractalComponent;
import org.objectweb.fractal.fraclet.annotation.generator.template.attribute.AttributeTemplate;
import org.objectweb.fractal.fraclet.annotation.processor.AttributeProcessor;

import spoon.reflect.declaration.CtClass;
import spoon.reflect.declaration.CtField;

/**
 * An annotation processor to manage the Attribute annotation. <br />
 * <br />
 * The <strong>only</strong> difference with the fraclet attribut processor is
 * the fact that an attribute can be inside a class annotated with the
 * {@link @DreamComponent} annotation
 * 
 * @author Pierre Garcia <pierre.garcia@inrialpes.fr>
 */
public class DreamAttributeProcessor extends AttributeProcessor
{

  /**
   * A constructor for the DreamAttributeProcessor
   * 
   * @param acAnnotation the Attribute annotation
   * @param processedField the CtField processed
   */
  @Override
  public void process(Attribute acAnnotation, CtField<?> processedField)
  {
    CtClass<?> ctclass = processedField.getParent(CtClass.class);
    if ((ctclass.getAnnotation(FractalComponent.class) != null)
        || (ctclass.getAnnotation(DreamComponent.class) != null))
      new AttributeTemplate(getFactory(), processedField);
  }
}
