/**
 * Dream
 * Copyright (C) 2003-2004 INRIA Rhone-Alpes
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Contact : dream@objectweb.org
 *
 * Initial developer(s): Pierre Garcia
 */

package org.objectweb.dream.dreamannotation.processor;

import java.util.List;

import org.objectweb.dream.dreamannotation.DreamMonolog;
import org.objectweb.dream.dreamannotation.generator.template.monolog.DreamMonologTemplate;
import org.objectweb.fractal.fraclet.annotation.annotations.FractalComponent;
import org.objectweb.fractal.fraclet.annotation.annotations.Service;

import spoon.processing.AbstractAnnotationProcessor;
import spoon.reflect.declaration.CtClass;
import spoon.reflect.declaration.CtElement;
import spoon.reflect.declaration.CtField;
import spoon.template.Substitution;
import spoon.template.Template;

/**
 * An annotation processor for the
 * 
 * @DreamMonolog annotation.
 * @author Pierre Garcia <Pierre.Garcia@inrialpes.fr>
 */
public class DreamMonologProcessor extends
		AbstractAnnotationProcessor<DreamMonolog, CtField<?>> {
	private static final boolean DEBUG = false;
	
	public void process(DreamMonolog logAnnotation, CtField<?> monologField) {
		// manage the @Monolog annotation
		if (DEBUG) System.out.println("[DEBUG] processing DreamMonolog annotation in "
		+ monologField.getParent(CtClass.class).getQualifiedName());
		CtClass<?> target = monologField.getParent(CtClass.class);

		if (target.getAnnotation(FractalComponent.class) != null) {
			System.err
					.println("\n### ERROR ### \nA field annotated with @DreamMonolog in the class "
							+ monologField.getParent(CtClass.class)
									.getQualifiedName()
							+ " can't be processed because this class is annotated with @FractalComponent.\n"
							+ "Two solutions depending on your needs \n"
							+ "- Replace @FractalComponent by @DreamComponent for a dream management of logger \n"
							+ "- Replace @DreamMonolog by @Monolog for a default fraclet management of logger \n"
							+ "###############\n");
			System.exit(1);
		}

		CtField<?> serviceField = getServiceField(target);

		if (serviceField == null) {
			System.err
					.println("\n### ERROR ### \nA field annotated with @Service must be present in the class \""
							+ monologField.getParent(CtClass.class)
									.getQualifiedName()
							+ "\" to make the logger work\n###############\n");
		} else {
			Template t = new DreamMonologTemplate(monologField, serviceField,
					logAnnotation);
			Substitution.insertAll(target, t);
		}
	}

	/**
	 * Get Service Field in this class or in super class
	 * 
	 * @param target
	 * @return
	 */
	private CtField<?> getServiceField(CtClass<?> target) {
		List<CtElement> serviceElement = null;
		CtField<?> serviceField = null;
		CtClass<?> currentClass = target;

		while (currentClass != null) {
			serviceElement = (List<CtElement>) target
					.getAnnotatedChildren(Service.class);
			if (serviceElement.size() > 0) {
				if (serviceElement.size() > 1) {
					System.err
							.println("Only one component service per component !");
				} else {
					return (CtField<?>) serviceElement.get(0);
				}
			}			
			if (currentClass.getSuperclass() != null){
			  currentClass = (CtClass<?>) currentClass.getSuperclass().getDeclaration();
			} else {
			  currentClass = null;
			}
		}
		return serviceField;
	}
}
